/***************************************************************************
 *            BinaryStressBase.h
 *
 *  Wed Jan 12 11:19:56 2005
 *  Copyright  2005  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _BINARYSTRESSBASE_H
#define _BINARYSTRESSBASE_H

/** \file
 * Binary StressBase class.
 */

#include "StressBase.h"

#include <boost/utility.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/scoped_array.hpp>

#include <string>

/** Provides access to binary stressbase files. 
 * The binary files are architecture dependant and should only be generated on
 * the same system where they are going to be used.
 */

class BinaryStressBase : public StressBase, public boost::noncopyable
{
  public:
    BinaryStressBase();
    BinaryStressBase (const std::string& file);

    void load (const std::string& file);
    
    virtual int get_accent (const std::string& name) const;

  private:
    int get_idx (const std::string& type) const;
    void allocate (const int new_size);

    unsigned size;
    boost::scoped_array<long long unsigned> keys;
    boost::scoped_array<unsigned char> accents;
};

#endif
