/***************************************************************************
 *            CmdLine.cc
 *
 *  Fri Mar  4 18:45:55 2005
 *  Copyright  2005  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
/** \file
 * Provide access to the cmdline parameters.
 */
 
#include "CmdLine.h"

/** Set the command line parameters for further use.
 * \param num main() argc parameter
 * \param params main() argv parameter
 */
 
void CmdLine::set (const int num, const char* const params[])
{
    singleton.reset (new CmdLine (num, params));
}

/** Get a reference to the command line parameters.
 * \return command line parameters
 */
 
const std::vector<std::string>& CmdLine::get()
{
    if (! singleton)
      singleton.reset (new CmdLine());
    return singleton->vals;
}

CmdLine::CmdLine (const int num, const char* const params[]) :
  vals (params, params + num)
{
}
    
boost::scoped_ptr<CmdLine> CmdLine::singleton;
