/***************************************************************************
 *            CopyPaste.h
 *
 *  Sat Jul  9 15:53:53 2005
 *  Copyright  2005  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _COPYPASTE_H
#define _COPYPASTE_H

#include <glibmm/refptr.h>

#include <boost/shared_ptr.hpp>

#include <vector>

class VocabTest;
    
namespace Gtk {
    class Clipboard;
}
    
class CopyPaste 
{
  public:
    CopyPaste (const boost::shared_ptr<VocabTest> vocab_test);
    void copy_structure (std::vector<std::pair<int, int> >& data);
    void copy_vocabs (const int chapter, const int section, const std::vector<int>& data);

    void paste_structure (const int chapter, const int section, const int vocab);
    void paste_vocabs (const int chapter, const int section, const int vocab);

  private:
    void paste (const int chapter, const int section, const int vocab);
  
    void append_chapter (const int chapter);
    void append_section (const int chapter, const int section);
    void append_vocab (const int chapter, const int section, const int vocab);

    void create_chapter (const int chapter, const bool synthetic = false);
    void create_section (const int chapter, const int section, const bool synthetic = false);
  
    void append_section_internal (const int chapter, const int section);
    void append_vocab_internal (const int chapter, const int section, const int vocab);
  
    void init_copy();
    void init_paste();
    void init();
    void done();
    void transfer_to_clipboard();

    bool transfer_from_clipboard();

    void scan_for_type();
  
    bool is_synthetic (const std::string& name) const;
    std::string strip_synthetic (const std::string& name) const;
  
    int last_target_chapter;
    int last_target_section;
    int last_target_vocab;

    int last_source_chapter;
    int last_source_section;
  
    enum {CHAPTERS, SECTIONS, VOCABS, NOTHING} type;
    
    boost::shared_ptr<VocabTest> source;
    boost::shared_ptr<VocabTest> target;
    
    boost::shared_ptr<VocabTest> copy;
    const boost::shared_ptr<VocabTest> vocab_test;
    std::vector<std::string> languages;
    const Glib::RefPtr<Gtk::Clipboard> clipboard;
    static const std::string synth;
};

#endif /* _COPYPASTE_H */
