/***************************************************************************
 *            EditTreeView.ch
 *
 *  Mon Oct 25 17:50:10 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _EDITTREEVIEW_H
#define _EDITTREEVIEW_H

#include "PopupTreeView.h"

/** \file
 * An editable Gtk::TreeView.
 */

/** Editable Gtk::TreeView.
 * It provides a hook for deletion and creation of entries as well as a rapid
 * insert mode. By pressing F2, as long as the user afterwards doesn't cancel
 * the editing, fields are advanced automatically and new entries are inserted
 * as needed. If an added entry is canceled in the first field, it is deleted
 * again immediately.
 *
 * \warning There are some known limits one which columns and renderes may be 
 * used for this to work:
 * - The first column has to be editable
 * - Only Gtk::CellRendererText and derived classes are accepted
 * - Only the first \e added renderer for a given column can be used for editing.
 */

class EditTreeView : public PopupTreeView
{
  public:
    EditTreeView (BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);

    /** Create a row at the Gtk::Treepath. */
    typedef sigc::signal<void,Gtk::TreePath, Gtk::TreeViewColumn*> type_signal_create_row_path;

    /** Delete a row at the Gtk::Treepath. */
    typedef sigc::signal<void,Gtk::TreePath, Gtk::TreeViewColumn*> type_signal_delete_row_path;

    type_signal_create_row_path signal_create_row_path();
    type_signal_delete_row_path signal_delete_row_path();

    int get_next_editable_column (const int old_idx) const;

  private:
    virtual bool on_key_press_event (GdkEventKey* event);
    virtual void on_editing_canceled();
    virtual void on_editing_done (const Glib::ustring& path, const Glib::ustring& new_text);

    type_signal_create_row_path signal_create_row_path_v;
    type_signal_delete_row_path signal_delete_row_path_v;

    void connect_signals();
    void disconnect_signals();
    void simulate_key (const int new_idx, const bool created = false);
    static bool is_editable_column (const Gtk::TreeViewColumn* c);
    int on_key_snoop (Gtk::Widget * where, GdkEventKey * event);

    sigc::connection old_editing_canceled;
    sigc::connection old_editing_done;
    sigc::connection old_key_snoop;
    GdkEventKey old_event;
    int old_idx;
    bool simulated_key;
    bool wasenter;
    bool newly_created;

    const Glib::RefPtr<Gnome::Glade::Xml> xml;
  
    static const int IDX_INIT        = -1;
    static const int IDX_TRY_NEW_ROW = -2;
    static const int IDX_TRY         = -3;
    static const int IDX_TRY_EDIT    = -4;
};


#endif /* _EDITTREEVIEW_H */
