/***************************************************************************
 *            IconStore.cc
 *
 *  Fri Nov 19 19:59:35 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/** \file
 * Provide icon managment.
 */

#include "ProgramPaths.h"
#include "IconStore.h"

#include "debug.h"

#include <gtkmm/iconfactory.h>
#include <boost/bind.hpp>
#include <boost/array.hpp>

/** Initialize local icons.
 * This registers a custom Gtk::IconFactory with the locally used icons.
 * Pixmaps are read from <tt>PACKAGE_PIXMAPS_DIR/name.png</tt>.
 */

IconStore::IconStore() :
  icon_factory (Gtk::IconFactory::create())
{
    const boost::array<char*, 2> names = {{VT_STOCK_PLAY, VT_STOCK_TIMER}};
    std::for_each (names.begin(), names.end(), boost::bind (&IconStore::register_icon, this, _1));

    icon_factory->add_default();
}

IconStore::~IconStore() 
{
    icon_factory->remove_default();
}

void IconStore::register_icon (const std::string& name)
{
    debug_log ("Loading icon in %1", name);

    try {
        Glib::RefPtr<Gdk::Pixbuf> icon (Gdk::Pixbuf::create_from_file (pix_fn (name)));
        if (icon)
          icon_factory->add (Gtk::StockID (name), Gtk::IconSet (icon));
        else
          debug_critical ("Icon %1 could not be loaded!", name);
    } catch (const Glib::Error& e) {
        debug_critical ("Icon %1 could not be loaded: %2", name, e.what());
    } catch (...) {
        debug_critical ("Icon %1 could not be loaded!", name);
    }
}

std::string IconStore::pix_fn (const std::string& name)
{
    const std::string path_name = ProgramPaths::pixmaps() + "/" + name + ".png";
    debug_log ("Loading icon in %1", path_name);
    return path_name;
}
