/***************************************************************************
 *            MainSession.h
 *
 *  Sat Dec  4 14:05:50 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _MAINSESSION_H
#define _MAINSESSION_H

#include "VocabTest.h"
#include "ApplicationSettings.h"
#include "TransientSettings.h"
#include "VocabSession.h"

#include <libglademm/xml.h>

class MainWindow;
    
class MainSession 
{
  public:
    MainSession (MainWindow* mw, const boost::shared_ptr<VocabTest> vocab_test);
    ~ MainSession();

    void start();
    void pause();
    void awake();
    void finish();
    void reset();
    void advance_and_show();
    void handle_input_guess();
    void handle_button_guess (const unsigned idx);
    void hide_results();

    void present_vocabulary_audio (const unsigned wait = 0);
    void present_vocabulary_audio_solution (const unsigned wait = 0);
  
    bool on_guess_key_press_event (GdkEvent* event);

  private:
    bool on_window_state_changed (GdkEventWindowState* event);
    bool on_timeout();

    void set_timer (const unsigned seconds);
    void clear_timer();
    void update_timer_mode();

    void advance();
  
    void present_results();
    void present_vocabulary (const bool silent = false);
    void present_vocabulary_written();
    void present_vocabulary_buttons();
    void present_solution (const bool solution, const bool right);
    void present_solution_audio (const unsigned wait);
    void handle_guess (const bool right);
  
    void be_annoying();
    void be_nice();
  
    bool lock_keyb() const;

    MainWindow *mw;
    sigc::connection iconify_connection;
    sigc::connection timeout_connection;
    sigc::connection grabkeyb_connection;
  
    // We handle the "use right answers instead of vocabs mode" manually
    unsigned right_answers_left;
    
    const Glib::RefPtr<Gnome::Glade::Xml> xml;
    const boost::shared_ptr<VocabTest> vocab_test;
    ApplicationSettings& settings;
    TransientSettings& tsettings;

    VocabSessionPtr session;
};

#endif /* _MAINSESSION_H */
