/***************************************************************************
 *            MainWindow.h
 *
 *  Mon Oct 18 12:25:08 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _MAINWINDOW_H
#define _MAINWINDOW_H

#include "VocabTest.h"
#include "VocabSession.h"
#include "EditTreeView.h"
#include "ApplicationSettings.h"
#include "TransientSettings.h"
#include "MainSession.h"
#include "TwoDeData.h"

#include <gtkmm/window.h>

#include <boost/shared_ptr.hpp>
#include <boost/multi_array.hpp>

#include <vector>
#include <string>

class VocabScoreDiagram;
class TrayIcon;
    
namespace Gtk {
    class Dialog;
    class Button;
    class SpinButton;
    class ComboBox;
    class MenuBar;
    class MenuItem;
    class HScrollbar;
    class VScrollbar;
    class TreeModelColumnRecord;
    class TreeView;
    class ListStore;
    class Toolbar;
    class ToolButton;
    class TextView;
    class Entry;
    class Notebook;
    class Table;
    class Image; 
    class Curve; 
};
    
class ResultListModelColumns : public Gtk::TreeModel::ColumnRecord
{
  public:
    ResultListModelColumns() { add(what); add(content); };

    Gtk::TreeModelColumn<Glib::ustring> what;
    Gtk::TreeModelColumn<Glib::ustring> content;
};

class VocabTreeModel;
class MarkerSonogram;
class SynthesisAnalysis;
class SynthesisDisplayLabel;
    
class MainWindow : public Gtk::Window 
{
  public:
    MainWindow (BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);

    // Menu
    void on_open_activate();
    void on_save_activate();
    void on_saveas_activate();
    void on_close_activate();
    void on_quit_activate();

    void on_cut_activate();
    void on_copy_activate();
    void on_paste_activate();
    void on_delete_activate();

    void on_addlang_activate();
    void on_fileproperties_activate();
    void on_settings_activate();

    void on_about_activate();
  
    void on_vocabcell_edited (const Glib::ustring& path, const Glib::ustring& new_text, const std::string language);
    void on_vocabtype_edited (const Glib::ustring& path, const Glib::ustring& new_text);
    void on_structure_edited (const Glib::ustring& path, const Glib::ustring& new_text);

    // Buttons
    void on_scoreerase_clicked();
    void on_scoreexpire_clicked();
    
    void on_showprev_clicked();
    void on_shownext_clicked();

    void on_play_audio();
    void on_play_audio_solution();
    void on_record_audio();
    void on_analyze_audio();
    
    void on_find();
    void on_find_change();

    void on_vocab_cut();
    void on_vocab_copy();
    void on_vocab_paste();
    void on_vocab_delete (const bool ask = true);
    void on_vocab_delete_path (Gtk::TreePath p, Gtk::TreeViewColumn* c);
    void on_vocab_add();
    void on_vocab_add_path (const Gtk::TreePath p, Gtk::TreeViewColumn* c);

    void on_structure_cut();
    void on_structure_copy();
    void on_structure_paste();
    void on_structure_delete (const bool ask = true);
    void on_structure_delete_path (Gtk::TreePath p, Gtk::TreeViewColumn* c);
    void on_structure_addsection();
    void on_structure_addchapter();

    void on_modified (const bool modified);

    bool on_search_equal (const Glib::RefPtr<Gtk::TreeModel>& model, int column, const Glib::ustring& key, const Gtk::TreeModel::iterator& iter);
    
    bool on_delete_event (GdkEventAny* event);
    void on_size_allocate (Gtk::Allocation& allocation);
    bool on_window_state_event (GdkEventWindowState* event);
    
    // Public UI helpers
    void get_selected_structure (std::vector<std::pair<int, int> >& data);
    void get_selected_vocabs (std::vector<int>& data);
    
    void set_question_button (const unsigned i, const std::string& s);

    void select_input();
    std::string get_guess_text();
    
    void create_pick_table (const unsigned num);

    void present_vocabulary_written (const std::string& what);
    void present_vocabulary_audio (const unsigned wait, const std::string& what, const std::string& language);
    void present_solution_written (const std::string& what, const bool audio = true);
    void present_solution_audio (const unsigned wait, const std::string& what, const std::string& language);
    
    void clear_result_list();
    void add_result_list_line (const std::string what, const std::string content);
    void hide_results();

    void handle_pending_events();
    void quiet_timer_mode();

  private:
    void open (const std::string& filename);

    void add_section (const int chapter, const int section);
    void add_chapter (const int chapter);

    void set_defaults();
  
    void present_manual_vocabulary_written();
    void present_manual_solution();

    bool handle_modified();

    // UI update
    void on_testmode_changed();
    void on_resultsshown_changed();
    void on_usetimer_changed();
    void recheck_ui();
  
    // UI helper
    void create_edit_structure (const bool force = true);
    void create_structure();

    bool show_find_box();
    bool hide_find_box();
    void find_next_match (Glib::RefPtr<VocabTreeModel> search, const bool skip);
    bool walk_model (Glib::RefPtr<Gtk::TreeModel> search, Gtk::TreePath& p);
    bool on_find_key_press_event (GdkEvent* event);
    void set_find_state (const bool error, const std::string& text);
    
    void set_question_box (const std::string& what);
    void set_solution_box (const std::string& what, const bool audio = true);

    int get_current_vocab (int& chapter, int& section, int& vocab) const;
    int get_treeview_vocab (Gtk::TreeView* tv, int& chapter, int& section, int& vocab) const;
    bool set_current_vocab (const int chapter, const int section, const int vocab);
    bool set_treeview_vocab (Gtk::TreeView* tv, const int chapter, const int section, const int vocab);
  
    // Sonogram
    typedef boost::multi_array<unsigned char, 3> ImageData;
    typedef boost::shared_ptr<ImageData> ImageDataPtr;
    typedef boost::multi_array_ref<unsigned char, 3> ImageDataRef;

    ImageDataPtr get_displayed (const TwoDeData::Data& sonogram) const;
    Glib::RefPtr<Gdk::Pixbuf> get_pixbuf (const ImageDataPtr A) const;
    void set_sonogram (TwoDeData::DataPtr raw_data, const bool user);
    void add_sonogram_label (const SynthesisDisplayLabel& label, const bool user);
    void on_play_voice (const bool user);
    void on_fit_changed();
    
    // Menu
    Gtk::MenuBar *menu_main;
    
    Gtk::MenuItem *menu_new;
    Gtk::MenuItem *menu_open;
    Gtk::MenuItem *menu_save;
    Gtk::MenuItem *menu_saveas;
    Gtk::MenuItem *menu_close;
    Gtk::MenuItem *menu_quit;
    Gtk::MenuItem *menu_cut;
    Gtk::MenuItem *menu_copy;
    Gtk::MenuItem *menu_paste;
    Gtk::MenuItem *menu_delete;
    Gtk::MenuItem *menu_find;
    Gtk::MenuItem *menu_addlang;
    Gtk::MenuItem *menu_fileproperties;
    Gtk::MenuItem *menu_settings;
    Gtk::MenuItem *menu_about;

    // Toolbar
    Gtk::Toolbar *tool_main;
    
    Gtk::ToolButton *tool_open;
    Gtk::ToolButton *tool_save;
    Gtk::ToolButton *tool_settings;
    Gtk::ToolButton *tool_fileproperties;
    Gtk::ToggleToolButton* tool_usetimer;

    // Search bar
    Gtk::Widget* findbox;
    Gtk::Entry* findvalue;
    Gtk::Button* find;
    Gtk::Label* findstate;
    
    // Left panel
    PopupTreeView *structuretree;
    Gtk::Menu *structuremenu;
    Gtk::MenuItem *structuremenu_cut;
    Gtk::MenuItem *structuremenu_copy;
    Gtk::MenuItem *structuremenu_paste;
    Gtk::MenuItem *structuremenu_delete;
    Gtk::MenuItem *structuremenu_addchapter;
    Gtk::MenuItem *structuremenu_addsection;

    Gtk::Button *starttraining;
    Gtk::Button *pausetraining;
    Gtk::Button *endtraining;

    // Notebook
    Gtk::Notebook *mainbook;
    
    // Edit page
    EditTreeView *vocabedit;
    Gtk::Menu *vocabmenu;
    Gtk::MenuItem *vocabmenu_cut;
    Gtk::MenuItem *vocabmenu_copy;
    Gtk::MenuItem *vocabmenu_paste;
    Gtk::MenuItem *vocabmenu_add;
    Gtk::MenuItem *vocabmenu_delete;

    Gtk::Button *addvocab;
    Gtk::Button *delvocab;

    // Training page
    Gtk::TreeView *resultlist;

    Gtk::Widget *questionbox;
    Gtk::Label *questionlabel;
    Gtk::Button *playagain;
    Gtk::Label *buttonquestionlabel;
    Gtk::Button *comparevoice;
    Gtk::Button *comparevoice1;

    Gtk::Widget *solutionbox;
    Gtk::Label *solutionlabel;
    Gtk::Button *playsolution;
    Gtk::Label *buttonsolutionlabel;

    Gtk::Widget *enterbox;
    Gtk::Entry *guess;
    Gtk::Button *guessready;
    Gtk::Button *showvocab;

    Gtk::Widget *pickbox;
    Gtk::Table *picktable;
    std::vector<Gtk::Button*> pickbuttons;
    Gtk::Button *showvocab2;

    Gtk::Widget *resultbuttons;
    Gtk::Button *confirmresult;

    Gtk::Widget *showbuttons;
    Gtk::Button *showprev;
    Gtk::Button *shownext;

    // File properties dialog
    Gtk::Dialog* fileproperties;
    Gtk::Entry* fileauthor;
    Gtk::TextView *filelicense;
    Gtk::TextView *filenotes;
    VocabScoreDiagram *scorecurve;

    // Settings dialog
    Gtk::Dialog *settingsdlg;    
    Gtk::Button *scoreerase;
    Gtk::Button *scoreexpire;

    // Voice analysis dialog
    Gtk::Dialog* analysisdlg;
    Gtk::Label* voicedata;
    MarkerSonogram *voiceref;
    MarkerSonogram *voiceuser;
    MarkerSonogram *channelusercorr;
    Gtk::HScrollbar *userh;
    Gtk::HScrollbar *refh;
    Gtk::VScrollbar *userv;
    Gtk::VScrollbar *refv;
    Gtk::Button *playuser;
    Gtk::Button *playref;
    
    boost::scoped_ptr<TrayIcon> tray_icon;

    ResultListModelColumns result_columns;
    Glib::RefPtr<Gtk::ListStore> result_store;

    // Various
    sigc::connection find_connection;
    const Glib::RefPtr<Gnome::Glade::Xml> xml;
    boost::scoped_ptr<SynthesisAnalysis> analysis;
    boost::shared_ptr<VocabTest> vocab_test;
    ApplicationSettings& settings;
    TransientSettings& tsettings;

    MainSession main_session;

    const static int PAGE_EDIT     = 0;
    const static int PAGE_TRAINING = 1;
    const static int PAGE_OPTIONS  = 2;
  };

#endif /* _MAINWINDOW_H */
