/***************************************************************************
 *            MarkerSonogram.h
 *
 *  Wed Jan 26 13:31:40 2005
 *  Copyright  2005  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _MARKERSONOGRAM_H
#define _MARKERSONOGRAM_H

#include <libgnomecanvasmm/canvas.h>

#include <libglademm.h>

namespace Gnome {
    namespace Canvas {
	class Group;
	class Pixbuf;
	class Line;
	class Points;
	class Text;
    }
}

class MarkerSonogram : public Gnome::Canvas::Canvas
{
  public:
    MarkerSonogram (BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
  
    void set_sonogram (Glib::RefPtr<Gdk::Pixbuf> sono);
    void clear_sonogram();
    void add_path (const Gnome::Canvas::Points& p, const std::string& color);
    void clear_paths();
    void show_paths (const bool visible = true);
    void add_label (const int x, const int y, const std::string& text);
    void add_fuzzy_label (const int x_min, const int y_min, const int x_max, const int y_max);
    void clear_labels();
    void show_labels (const bool visible = true);

    void set_lock (const unsigned lock_x, const unsigned lock_y);
    void set_turned (const bool rotation = true);
    void set_zoom_x (const double zoom);
    void set_zoom_y (const double zoom);
    void set_fit (const bool fit);
  
  protected:
    void recalc_display (const unsigned width, const unsigned height);
    void recalc_display_current();

    virtual void on_size_allocate (Gtk::Allocation& allocation);
    virtual bool on_event (GdkEvent* event);
    Gnome::Art::AffineTrans get_correction() const;

  private:
    const Glib::RefPtr<Gnome::Glade::Xml> xml;
      
    Gnome::Canvas::Pixbuf* pixbuf;
    Gnome::Canvas::Group* parent;
    std::vector<Gnome::Canvas::Item*> labelstuff;
    std::vector<Gnome::Canvas::Line*> pathstuff;
    unsigned lock_x, lock_y;
    double zoom_x, zoom_y;
    double real_width, real_height;
    bool fit;
    bool rotation;
    bool hpath, hlabel;

    double motion_x, motion_y;
    int orig_x, orig_y;
};

#endif /* _MARKERSONOGRAM_H */
