/***************************************************************************
 *            PopupTreeView.cc
 *
 *  Mon Oct 25 17:50:10 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
/** \file
 * An Gtk::TreeView with DEL and context menu support.
 */

#include "PopupTreeView.h"

#include "debug.h"

#include <gtkmm/menu.h>
#include <gtkmm/window.h>
#include <gdk/gdkkeysyms.h>

#include <algorithm>

/** Initialize with libglade.
 */

PopupTreeView::PopupTreeView (BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
  : Glib::ObjectBase (typeid(PopupTreeView)), 
    Gtk::TreeView (cobject),
    xml (refGlade),
    menu (NULL)
{
}

/** Set the popup menu used on right click.
 * \param m Context menu
 */

void PopupTreeView::set_popup (Gtk::Menu* m)
{
    menu = m;
}
  
bool PopupTreeView::on_key_press_event (GdkEventKey* event)
{
    switch (event->keyval) {
      case GDK_Delete:
      case GDK_KP_Delete:
        signal_delete_row_v.emit ();
        break;

      default:
        return TreeView::on_key_press_event (event);
    }

    return true;
}

bool PopupTreeView::on_button_press_event (GdkEventButton* event)
{
    if ((event->type == GDK_BUTTON_PRESS) && 
        (event->button == 3) &&
	(menu != NULL)) {
	Gtk::TreePath p; 
	int cell_x, cell_y;
	Gtk::TreeViewColumn* c;
	if (get_path_at_pos ((int) event->x, (int) event->y, p, c, cell_x, cell_y) &&
	    ! get_selection()->is_selected (p)) {
	    TreeView::on_button_press_event (event);
	}
	menu->popup (event->button, event->time);
	return true;
    }
    return TreeView::on_button_press_event (event);
}

/** Connect to this signal to provide the delete functionality.
 * It is called when the current row needs to be deleted.
 */

PopupTreeView::type_signal_delete_row PopupTreeView::signal_delete_row()
{
    return signal_delete_row_v;
}
