/***************************************************************************
 *            PopupTreeView.ch
 *
 *  Mon Oct 25 17:50:10 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _POPUPTREEVIEW_H
#define _POPUPTREEVIEW_H

/** \file
 * An Gtk::TreeView with DEL and context menu support.
 */

#include <libglademm.h>

#include <gtkmm/treeview.h>

#ifndef DOXYGEN_SHOULD_SKIP_THIS

namespace Gtk {
    class Menu;
}

#endif

/** Gtk::TreeView with DEL and context menu support.
 * It provides a hook for the deletion of the current entry and the possibility
 * to specify a context menu which will be shown on right click.
 */

class PopupTreeView : public Gtk::TreeView
{
  public:
    PopupTreeView (BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);

    void set_popup (Gtk::Menu* m);
  
    /** Delete the current row. */
    typedef sigc::signal<void> type_signal_delete_row;

    type_signal_delete_row signal_delete_row();

  protected:
    virtual bool on_key_press_event (GdkEventKey* event);
    virtual bool on_button_press_event (GdkEventButton* event);

    type_signal_delete_row signal_delete_row_v;

  private:
    const Glib::RefPtr<Gnome::Glade::Xml> xml;
    Gtk::Menu* menu;
};


#endif /* _POPUPTREEVIEW_H */
