/***************************************************************************
 *            ProgramPaths.cc
 *
 *  Tue May  3 17:23:27 2005
 *  Copyright  2005  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
/** \file
 * Cross platform settings and program path management.
 */

#include "ProgramPaths.h"
#include "CmdLine.h"

#include "debug.h"

#include "glibmm/miscutils.h"

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

const std::string ProgramPaths::get_locale_dir() const
{
    #ifdef WIN32
    return win32_home() + "/share/locale";
    #else
    return LOCALEDIR;
    #endif
}

const std::string ProgramPaths::get_home_dir() const
{
    std::string result;
    try {
	result = Glib::get_home_dir();
    } catch (...) {
	debug_log ("No home directory found, using fallback");
	result = "";
    }
    #ifdef WIN32
    if (result == "")
      result = win32_home();
    #endif
    return result;
}

const std::string ProgramPaths::get_data_dir() const
{
    #ifdef WIN32
    return win32_home() + "/share/" + domain();
    #else
    return std::string (PKGDATADIR);
    #endif
}

const std::string ProgramPaths::get_pixmaps_dir() const
{
    #ifdef WIN32
    return win32_home() + "/share/pixmaps";
    #else
    return std::string (PIXMAPDIR);
    #endif
}

const std::string ProgramPaths::get_gettext_name() const
{
    return PACKAGE;
}

const std::string ProgramPaths::get_glade_name() const
{
    return data() + "/" + domain() + ".glade";
}

#ifdef WIN32
const std::string& ProgramPaths::win32_home() const
{
    if (win32_program_dir == "") {
	const std::vector<std::string>& params = CmdLine::get();
	if (params.empty()) {
	    debug_log ("Couldn't load program path");
	} else {
	    // Strip executable and first path name (should be .../bin/whatever.exe)
	    win32_program_dir = Glib::path_get_dirname (Glib::path_get_dirname (params.at (0)));
	    debug_log ("Program path: %1", win32_program_dir);
	}
    }
    return win32_program_dir;
}
#endif
