/***************************************************************************
 *            ProgramPaths.h
 *
 *  Tue May  3 16:31:42 2005
 *  Copyright  2005  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _PROGRAMPATHS_H
#define _PROGRAMPATHS_H

/** \file
 * Cross platform settings and program path management.
 */

#include "RegisteredSingleton.h"

#include <string>

/** Singleton to provide cross platform settings and program path management.
 */

class ProgramPaths: public RegisteredSingleton<ProgramPaths, 98>
{
  public:
    static const std::string locale() { return ProgramPaths::get().get_locale_dir(); };
    static const std::string home() { return ProgramPaths::get().get_home_dir(); };
    static const std::string data() { return ProgramPaths::get().get_data_dir(); };
    static const std::string pixmaps() { return ProgramPaths::get().get_pixmaps_dir(); };

    // These are not really directories
    static const std::string domain() { return ProgramPaths::get().get_gettext_name(); };
    static const std::string glade() { return ProgramPaths::get().get_glade_name(); };
  
  private:
    const std::string get_locale_dir() const;
    const std::string get_home_dir() const;
    const std::string get_data_dir() const;
    const std::string get_pixmaps_dir() const;
  
    const std::string get_gettext_name() const;
    const std::string get_glade_name() const;
  
    #ifdef WIN32
    const std::string& win32_home() const;
    mutable std::string win32_program_dir;
    #endif
};

#endif /* _PROGRAMPATHS_H */
