/***************************************************************************
 *            ScoreDiagram.cc
 *
 *  Wed Jan 26 13:31:46 2005
 *  Copyright  2005  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "ScoreDiagram.h"

#include "debug.h"

#include <libgnomecanvasmm/group.h>
#include <libgnomecanvasmm/rect.h>
#include <libgnomecanvasmm/affinetrans.h>

#include <boost/checked_delete.hpp>

ScoreDiagram::ScoreDiagram (BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade) :
  Glib::ObjectBase (typeid (ScoreDiagram)),
  Gnome::Canvas::Canvas (cobject),
  xml (refGlade),
  real_width (1), real_height (1)
{
    background = Gtk::manage (new Gnome::Canvas::Rect (*root(), 0, 0, 1, 1));
    background->property_fill_color() = "black";
    create_grid();
    lines_parent = Gtk::manage (new Gnome::Canvas::Group (*root()));
}

void ScoreDiagram::on_size_allocate (Gtk::Allocation& allocation)
{
    if (get_width()  != allocation.get_width() ||
	get_height() != allocation.get_height())
      recalc_display (allocation.get_width(), allocation.get_height());
    Gnome::Canvas::Canvas::on_size_allocate (allocation);
}

void ScoreDiagram::add_line (const Gnome::Canvas::Points& p, const std::string& color)
{
    debug_log ("Loading path with %1 points", p.size());
    if (p.size() <= 1) return;
    Gnome::Canvas::Line* l = Gtk::manage (new Gnome::Canvas::Line (*lines_parent, p));
    l->property_fill_color() = color;
    l->property_width_pixels() = 2;
    lines.push_back (l);
}

void ScoreDiagram::clear_lines()
{
    std::for_each (lines.begin(), lines.end(), boost::checked_deleter<Gnome::Canvas::Item>());
    lines.clear();
}

void ScoreDiagram::create_grid()
{    
    const unsigned num = 10;
    grid_parent = Gtk::manage (new Gnome::Canvas::Group (*root()));
    Gnome::Canvas::Points p (2);
    for (unsigned i = 0; i < num + 1; ++i) {
	p.clear();
	p.push_back (Gnome::Art::Point (0, float (i) / num));
	p.push_back (Gnome::Art::Point (1, float (i) / num));
	add_grid_line (*grid_parent, p, i % 5 == 0);
    }
    for (unsigned i = 0; i < num + 1; ++i) {
	p.clear();
	p.push_back (Gnome::Art::Point (float (i) / num, 0));
	p.push_back (Gnome::Art::Point (float (i) / num, 1));
	add_grid_line (*grid_parent, p, i % 5 == 0);
    }
}

void ScoreDiagram::add_grid_line (Gnome::Canvas::Group& parent, 
  const Gnome::Canvas::Points& p, const bool thick)
{
    Gnome::Canvas::Line* l = Gtk::manage (new Gnome::Canvas::Line (parent, p));
    l->property_fill_color() = "gray";
    if (thick)
      l->property_width_pixels() = 2;
    grid.push_back (l);
}

void ScoreDiagram::recalc_display_current()
{
    recalc_display (get_width(), get_height());
}

void ScoreDiagram::recalc_display (const unsigned width, const unsigned height)
{
    const double scale_x = (width - 10) / real_width;
    const double scale_y = (height - 10) / real_height;

    background->property_x1() = -5;
    background->property_y1() = -5;
    background->property_x2() = width - 5;
    background->property_y2() = height - 5;
    lines_parent->affine_absolute (Gnome::Art::AffineTrans::scaling (scale_x, scale_y));
    grid_parent->affine_absolute (Gnome::Art::AffineTrans::scaling (
      scale_x * real_width, scale_y * real_height));
    
    set_scroll_region (-5, -5, width - 5, height - 5);
}
