/***************************************************************************
 *            ScoreDiagram.h
 *
 *  Wed Jan 26 13:31:40 2005
 *  Copyright  2005  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _SCOREDIAGRAM_H
#define _SCOREDIAGRAM_H

#include <libgnomecanvasmm/canvas.h>
#include <libgnomecanvasmm/line.h>

#include <libglademm.h>

namespace Gnome {
    namespace Canvas {
	class Group;
	class Rect;
    }
}

class ScoreDiagram : public Gnome::Canvas::Canvas
{
  public:
    ScoreDiagram (BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
  
    template <typename iter> void set_points (const iter& begin, const iter& end);
  
  protected:
    void recalc_display (const unsigned width, const unsigned height);
    void recalc_display_current();

    virtual void on_size_allocate (Gtk::Allocation& allocation);
  
    void add_line (const Gnome::Canvas::Points& p, const std::string& color);
    void clear_lines();

    void create_grid();
    void add_grid_line (Gnome::Canvas::Group& parent, 
      const Gnome::Canvas::Points& p, const bool thick);

  protected:
    const Glib::RefPtr<Gnome::Glade::Xml> xml;
  
  private:    
    Gnome::Canvas::Group* lines_parent;
    Gnome::Canvas::Group* grid_parent;
    Gnome::Canvas::Rect* background;
    std::vector<Gnome::Canvas::Line*> lines;
    std::vector<Gnome::Canvas::Line*> grid;
    double real_width, real_height;
};

template <typename iter> void ScoreDiagram::set_points (const iter& begin, const iter& end)
{
    clear_lines();
    Gnome::Canvas::Points p;
    int count = 0;
    for (iter i = begin; i != end; ++i, ++count)
      p.push_back (Gnome::Art::Point (count, 1 - *i));
    add_line (p, "green");
    real_width = count;
    recalc_display_current();
}

#endif /* __SCOREDIAGRAM_H */
