/***************************************************************************
 *            SessionFeeder.h
 *
 *  Fri Dec 10 12:22:18 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _SESSIONFEEDER_H
#define _SESSIONFEEDER_H

#include "VocabTest.h"
#include "VocabCache.h"
#include "SessionVocab.h"

#include <string>
#include <vector>

struct SessionFeederSection {
    SessionFeederSection (const unsigned chapter, const unsigned section);
    bool operator< (const SessionFeederSection& other) const;

    unsigned chapter, section;
};

struct SessionFeederVocab {
    SessionFeederVocab (const unsigned chapter, const unsigned section, const unsigned vocab);
    bool operator< (const SessionFeederVocab& other) const;

    unsigned chapter, section, vocab;
};

class VocabSession;

class SessionFeeder;
    
typedef boost::shared_ptr<SessionFeeder> SessionFeederPtr;

class SessionFeeder
{
  public:
    static SessionFeederPtr create (const boost::shared_ptr<VocabTest> vocab_test);
  
    void set_languages (const std::string& first, const std::string& second);
    void set_ors (const std::string& first, const std::string& second);
    void set_mode (const bool random, const bool scorebased, const unsigned first_repeat, const unsigned wrong_repeat);
    void set_alternatives (const unsigned num, const bool sorted, const bool samepreferred);
  
    void add_chapter (const unsigned chapter);
    void add_section (const unsigned chapter, const unsigned section);
  
  private:
    SessionFeeder (const boost::shared_ptr<VocabTest> vocab_test);
    
    bool get (SessionVocabPtr& vocab);
    unsigned size() const;

    // List preparation
    void start();
  
    void append_chapter (const unsigned chapter);
    void append_section (const unsigned chapter, const unsigned section);
    void append_vocab (const unsigned chapter, const unsigned section, const unsigned vocab);
  
    bool test_score (const SessionFeederVocab& vocab) const;
  
    typedef std::set<unsigned> ChaptersList;
    typedef std::set<SessionFeederSection> SectionsList;
    typedef std::set<SessionFeederVocab> VocabsList;
    ChaptersList chapters;
    SectionsList sections;
    VocabsList vocabs;

    typedef std::vector<SessionFeederVocab> FinalsList;
    FinalsList finals;

    void get_alternatives (const unsigned chapter, const unsigned section, const unsigned vocab, std::vector<Glib::ustring>& alternatives) const;
    SessionVocabPtr SessionFeeder::prepare_vocab (const unsigned chapter, const unsigned section, const unsigned vocab) const;

    bool started;
    bool random_order;
    bool score_based;
    unsigned alternatives_num;
    unsigned first_repeat_num;
    unsigned wrong_repeat_num;
    bool alternatives_sorted;
    bool alternatives_samepreferred;
    std::string first_language;
    std::string second_language;
    std::string first_or;
    std::string second_or;

    const boost::shared_ptr<VocabTest> vocab_test;
    const VocabCache& vocab_cache;
    
  friend class VocabSession;
};

#endif /* _SESSIONFEEDER_H */
