/***************************************************************************
 *            SessionVocab.cc
 *
 *  Mon Nov 29 12:20:08 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "SessionVocab.h"
#include "VocabChecker.h"
#include "VocabCache.h"

SessionVocab::SessionVocab (
  const Glib::ustring& first_formatted, const Glib::ustring& first_short, const Glib::ustring& first_audio,
  const Glib::ustring& second_formatted, const Glib::ustring& second_clear, const Glib::ustring& second_audio, 
  const std::vector<Glib::ustring>& second_buttons, const Glib::ustring& second_button_right,
  const unsigned first_repeat_num, const unsigned wrong_repeat_num, const unsigned id) :
  first_formatted (first_formatted), 
  first_short (first_short), 
  first_audio (first_audio), 
  second_formatted (second_formatted), 
  second_clear (second_clear), 
  second_audio (second_audio), 
  second_buttons (second_buttons),
  second_button_right (second_button_right),
  repeat_num (first_repeat_num),
  wrong_repeat_num (wrong_repeat_num),
  id (id),
  answered_right (0),
  answered_wrong (0),
  answered_unknown (0)
{
}

Glib::ustring SessionVocab::get_question() const
{
    return first_formatted;
}

Glib::ustring SessionVocab::get_question_audio() const
{
    return first_audio;
}

Glib::ustring SessionVocab::get_question_short() const
{
    return first_short;
}

Glib::ustring SessionVocab::get_solution() const
{
    return second_formatted;
}

Glib::ustring SessionVocab::get_solution_audio() const
{
    return second_audio;
}

Glib::ustring SessionVocab::get_alternative (const unsigned index) const
{
    return index < second_buttons.size() ? second_buttons[index] : "";
}
    
bool SessionVocab::check (const Glib::ustring& test) const
{
    return VocabChecker::check (test, second_clear);
}

bool SessionVocab::check (const unsigned index) const
{
    return second_button_right == second_buttons[index];
}

void SessionVocab::answered (bool right) 
{
    // If wrong, ask again, increment by one because it is decremented in defer
    if (! right && wrong_repeat_num + 1 > repeat_num) repeat_num = wrong_repeat_num + 1;
    if (right) {
        answered_right++;
	VocabCache::get().right (id);
    } else {
        answered_wrong++;
	VocabCache::get().wrong (id);
    }
}

void SessionVocab::unanswered()
{
    answered_unknown++;
    // Not knowing is bad :-)
    VocabCache::get().wrong (id);
}
