/***************************************************************************
 *            SessionVocab.h
 *
 *  Mon Nov 29 12:18:35 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _SESSIONVOCAB_H
#define _SESSIONVOCAB_H

#include <glibmm/ustring.h>

#include <boost/shared_ptr.hpp>

#include <vector>

class VocabSession;

class SessionVocab
{
  public:
    SessionVocab (
      const Glib::ustring& first_formatted, const Glib::ustring& first_short, const Glib::ustring& first_audio,
      const Glib::ustring& second_formatted, const Glib::ustring& second_clear, const Glib::ustring& second_audio,
      const std::vector<Glib::ustring>& second_buttons, const Glib::ustring& second_button_right,
      const unsigned first_repeat_num, const unsigned wrong_repeat_num,
      const unsigned id);
      
  
    Glib::ustring get_question() const;
    Glib::ustring get_question_short() const;
    Glib::ustring get_question_audio() const;
    Glib::ustring get_solution() const;
    Glib::ustring get_solution_audio() const;
    Glib::ustring get_alternative (const unsigned index) const;
    
    bool check (const Glib::ustring& test) const;
    bool check (const unsigned index) const;

    void answered (bool right);
    void unanswered();
  
  private:
    // Text data for presentation and checking
    const Glib::ustring first_formatted;
    const Glib::ustring first_short;
    const Glib::ustring first_audio;
    const Glib::ustring second_formatted;
    const Glib::ustring second_clear;
    const Glib::ustring second_audio;
    const std::vector<Glib::ustring> second_buttons;
    const std::string second_button_right;
    unsigned repeat_num; // This one gets decremented
    const unsigned wrong_repeat_num;
    const unsigned id;

    // Statistics
    unsigned answered_right;
    unsigned answered_wrong;
    unsigned answered_unknown;
  
  friend class VocabSession;
};

typedef boost::shared_ptr<SessionVocab> SessionVocabPtr;

#endif /* _SESSIONVOCAB_H */
