/***************************************************************************
 *            SettingsAdaptors.cc
 *
 *  Wed Feb 23 13:21:48 2005
 *  Copyright  2005  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include "SettingsAdaptors.h"

#include "ucompose.hpp"
#include "debug.h"

#include <gtkmm/menu.h>
#include <gtkmm/treeview.h>

#include <boost/utility.hpp>

void MenuMRUAdaptor::add_entry (const unsigned pos, const std::string& value) 
{
    debug_assert (widget != NULL);
    if (! widget->has_submenu())
      widget->set_submenu (*Gtk::manage (new Gtk::Menu()));
    
    using namespace Gtk::Menu_Helpers;
    
    MenuList items = widget->get_submenu()->items(); 
    MenuList::iterator i = boost::next (items.begin(), pos);
    items.insert (i, MenuElem (value, sigc::bind<std::string> (signal_v.make_slot(), value)));
    
    widget->set_sensitive();
};

void MenuMRUAdaptor::del_entry (const unsigned pos) 
{
    debug_assert (widget != NULL);
    if (! widget->has_submenu()) return;
	
    using namespace Gtk::Menu_Helpers;
    
    MenuList items = widget->get_submenu()->items(); 
    MenuList::iterator i = boost::next (items.begin(), pos);
    items.erase (i);
    
    widget->set_sensitive (items.size() > 0);
};

void TreeViewMRUAdaptor::link_widget (const Settings& settings)
{
    debug_log ("%1: Getting the XML widget", widget_name);
    settings.xml->get_widget (widget_name, widget);
    columns.reset (new ModelColumns);
    store = Gtk::ListStore::create (*columns);
    widget->set_model (store);
    widget->append_column ("", columns->text);
    widget->signal_row_activated().connect (sigc::mem_fun (this, &TreeViewMRUAdaptor::signal_activated));
}


void TreeViewMRUAdaptor::signal_activated (const Gtk::TreePath& path, Gtk::TreeViewColumn* column)
{
    signal_v.emit ((*store->get_iter (path))[columns->file]);
}

void TreeViewMRUAdaptor::add_entry (const unsigned pos, const std::string& value) 
{
    Gtk::TreeIter i = store->insert (boost::next (store->children().begin(), pos));
    (*i)[columns->file] = value;
    (*i)[columns->text] = strmanip::ucompose ("%1 (%2)", Glib::path_get_basename (value),
      Glib::path_get_dirname (value));
}

void TreeViewMRUAdaptor::del_entry (const unsigned pos) 
{
    store->erase (boost::next (store->children().begin(), pos));
}
