/***************************************************************************
 *            SettingsHelper.cc
 *
 *  Wed Feb 23 13:12:02 2005
 *  Copyright  2005  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "SettingsHelper.h"

#include "ucompose.hpp"

void MRUSetting::add_mru (const std::string& value)
{
    const std::string file_name = Glib::path_is_absolute (value) ?
      value : Glib::build_filename (Glib::get_current_dir(), value);
	
    valuelist_type::iterator old = std::find (values.begin(), values.end(), file_name);
    if (old != values.end()) {
	widget->del_entry (std::distance (values.begin(), old));
        values.erase (old);
    }
    while (values.size() > max_num - 1) {
    	widget->del_entry (values.size() - 1);
        values.resize (values.size() - 1);
    }

    widget->add_entry (0, file_name);
    values.insert (values.begin(), file_name);    
    
    std::string item;
    while ((item = settings.manager->get_name (group, 0)) != "")
      settings.manager->delete_entry (group, item);
    settings.manager->set_unsigned (group, "number", values.size());
    valuelist_type::const_iterator i;
    unsigned idx;
    for (idx = 1, i = values.begin(); i != values.end(); ++idx, ++i)
      settings.manager->set_string (group, strmanip::ucompose ("entry_%1", idx), *i);
}

void MRUSetting::status_to_ui()
{
    debug_log ("%1: Promoting saved value to UI", group);
    values.clear();
    unsigned number;
    if (settings.manager->get_checked (group, "number", number)) {
	if (number > max_num)
	  number = max_num;
	for (unsigned i = 1; i <= number; ++i)
	  values.push_back (settings.manager->get_string (group, strmanip::ucompose ("entry_%1", i)));
    }
    
    unsigned idx = 0;
    valuelist_type::const_iterator i;
    for (i = values.begin(); i != values.end(); ++i, ++idx)
      widget->add_entry (idx, *i);
}
