/***************************************************************************
 *            SettingsManager.h
 *
 *  Mon Nov 29 19:04:05 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _SETTINGSMANAGER_H
#define _SETTINGSMANAGER_H

#include <sigc++/signal.h>

#include <boost/shared_ptr.hpp>

#include <string>
#include <sstream>

class SettingsManager;
    
typedef boost::shared_ptr<SettingsManager> SettingsManagerPtr;

class SettingsManager 
{
  public:
    SettingsManager();
    virtual ~ SettingsManager();
  
    virtual int get_int (const std::string& group, const std::string& key) const;
    virtual unsigned get_unsigned (const std::string& group, const std::string& key) const;
    virtual float get_float (const std::string& group, const std::string& key) const;
    virtual std::string get_string (const std::string& group, const std::string& key) const;
    virtual bool get_bool (const std::string& group, const std::string& key) const;
  
    virtual bool get_checked_int (const std::string& group, const std::string& key, int& value) const;
    virtual bool get_checked_unsigned (const std::string& group, const std::string& key, unsigned& value) const;
    virtual bool get_checked_float (const std::string& group, const std::string& key, float& value) const;
    virtual bool get_checked_bool (const std::string& group, const std::string& key, bool& value) const;
    virtual bool get_checked_string (const std::string& group, const std::string& key, std::string& value) const = 0;

    virtual void set_int (const std::string& group, const std::string& key, const int value);
    virtual void set_unsigned (const std::string& group, const std::string& key, const unsigned value);
    virtual void set_float (const std::string& group, const std::string& key, const float value);
    virtual void set_bool (const std::string& group, const std::string& key, const bool value);
    virtual void set_string (const std::string& group, const std::string& key, const std::string& value) = 0;

    virtual void set_description (const std::string& group, const std::string& key, const std::string& value) = 0;
    virtual void set_hint (const std::string& group, const std::string& key, const std::string& value) = 0;

    template <class T> bool get_checked (const std::string& group, const std::string& key, T& value) const;
    template <class T> void set (const std::string& group, const std::string& key, const T& value);

    virtual bool delete_entry (const std::string& group, const std::string& key) = 0;
    virtual std::string get_name (const std::string& group, const unsigned idx) = 0;
    
    typedef sigc::signal<void> signal_type;
    virtual signal_type get_signal (const std::string& group, const std::string& key) = 0;
};

template <class T>
bool SettingsManager::get_checked (const std::string& group, const std::string& key, T& value) const
{
    std::string newval;
    if (get_checked_string (group, key, newval)) {
	std::istringstream s (newval);
	s >> value;
	return true;
    } else return false;
}

template <class T>
void SettingsManager::set (const std::string& group, const std::string& key, const T& value)
{
    std::ostringstream s;
    s << value;
    set_string (group, key, s.str());
}

#endif /* _SETTINGSMANAGER_H */
