/***************************************************************************
 *            SynthesisAnalysis.h
 *
 *  Mon Mar 21 16:43:02 2005
 *  Copyright  2005  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _SYNTHESISANALYSIS_H
#define _SYNTHESISANALYSIS_H

#include "TwoDeData.h"

#include <glibmm/spawn.h>

class SynthesisDisplayLabel
{
  public:
    SynthesisDisplayLabel() : posx (0), posy (0) {};
    SynthesisDisplayLabel (const int posx, const int posy)
      : posx (posx), posy (posy) {};
    SynthesisDisplayLabel (const int posx, const int posy, const std::string& label)
      : posx (posx), posy (posy), label (label) {};
    int posx, posy;
    std::string label;
};

typedef std::vector<SynthesisDisplayLabel> SynthesisDisplayLabelData;

class SynthesisEvaluationEntry
{
  public:
    SynthesisEvaluationEntry () :
      ref_from (0), ref_to (0), 
      user_from (0), user_to (0), 
      len_score (0), energy_score (0) {};
    operator const std::string() const;
    std::string label;
    double ref_from, ref_to;
    double user_from, user_to;
    double len_score, energy_score;
};

typedef std::vector<SynthesisEvaluationEntry> SynthesisEvaluationEntryData;

class SynthesisAnalysis
{
  public:
    SynthesisAnalysis();
    ~SynthesisAnalysis();
  
    void analyze();
    void play_user() const;
    void play_ref() const;
    std::string get_text() const;
  
    TwoDeData::DataPtr get_user() const { return user; }
    TwoDeData::DataPtr get_ref() const { return ref; };
    const SynthesisDisplayLabelData& get_user_labels() const { return user_labels; };
    const SynthesisDisplayLabelData& get_ref_labels() const { return ref_labels; };
  private:
    void shutdown();
    void get_file (const std::string& name, TwoDeData::DataPtr& data, 
      SynthesisDisplayLabelData& labels) const;
    void get_errors (const std::string& name, 
      SynthesisEvaluationEntryData& data) const;
    void play (const std::string& name, const double from, const double to) const;
    bool was_shutdown;
    Glib::Pid pid;
  
    TwoDeData::DataPtr user;
    TwoDeData::DataPtr ref;
    SynthesisDisplayLabelData user_labels;
    SynthesisDisplayLabelData ref_labels;
    SynthesisEvaluationEntryData errors;
};

#endif /* _SYNTHESISANALYSIS_H */
