/***************************************************************************
 *            SynthesisInterface.cc
 *
 *  Thu Jan 13 13:02:43 2005
 *  Copyright  2005  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include "debug.h"

#include "SynthesisInterface.h"
#include "ApplicationSettings.h"

bool SynthesisInterface::available (const std::string& language) const
{
    ApplicationSettings& settings = ApplicationSettings::get();
    std::string proggi = settings.get_synthesis_program (language);
    std::string params = settings.get_synthesis_params (language);
    return proggi != "" || params != "";
}

void SynthesisInterface::play (const unsigned delay, const std::string& text, const std::string& language)
{
    update_language (language);
    if (data[language]) {
	data[language]->play (delay, text);
    }
}
  
void SynthesisInterface::update_language (const std::string& language)
{
    ApplicationSettings& settings = ApplicationSettings::get();
    std::string proggi = settings.get_synthesis_program (language);
    std::string params = settings.get_synthesis_params (language);
    const StringMap::const_iterator old_prog = old_paths.find (language);
    const StringMap::const_iterator old_param = old_params.find (language);
    try {
	if (old_prog == old_paths.end() || old_param == old_params.end() ||
	    proggi != old_prog->second || params != old_param->second) {
	    debug_log ("(Re)loading synthesis engine for \"%1\": %2 %3", language, proggi, params);
	    data[language] = SynthesisProgramPtr (new SynthesisProgram (proggi, params));
	    old_paths[language] = proggi;
	    old_params[language] = params;
	}
	if (settings.is_synthesis_timeout()) {
	    timeouts[language].disconnect();
	    timeouts[language] = Glib::signal_timeout().connect (sigc::bind<std::string> (sigc::mem_fun (this, &SynthesisInterface::timeout), 
	      language), settings.get_synthesis_timeout() * 1000);
	}
    } catch (...) {
	debug_log ("Failed to execute program for \"%1\"", language);
	timeout (language);
    }
}

bool SynthesisInterface::timeout (const std::string& language)
{
    old_paths.erase (language);
    old_params.erase (language);
    timeouts.erase (language);
    data.erase (language);
    return false;
}
