/***************************************************************************
 *            SynthesisProgram.cc
 *
 *  Thu Jan 13 13:14:38 2005
 *  Copyright  2005  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "debug.h"

#include "SynthesisProgram.h"
#include "debug.h"

#include <glibmm/shell.h>
#include <glibmm/spawn.h>
#include <glibmm/main.h>

#include <vector>

#include <unistd.h>

SynthesisProgram::SynthesisProgram() : 
  prog_pipe (0)
{
}

SynthesisProgram::SynthesisProgram (const std::string& file, const std::string& params) :
  prog_pipe (0)
{
    try {
	run (file, params);
    } catch (...) {
	// The destructor is not executed
	terminate();
	throw;
    }
}
     
SynthesisProgram::~SynthesisProgram() {
    terminate();
}

void SynthesisProgram::run (const std::string& file, const std::string& params) 
{
    debug_log ("Running program: %1", file);

    terminate();
    
    std::vector<std::string> paramv;
    if (params != "")
      paramv = Glib::shell_parse_argv (params);

    paramv.insert (paramv.begin(), file);

    // This may throw
    int prog_stdin = 1;
    Glib::spawn_async_with_pipes (".", paramv,
      Glib::SpawnFlags (0), sigc::slot<void>(), 0, &prog_stdin);
    prog_pipe = prog_stdin;
}

void SynthesisProgram::play (const unsigned delay, const std::string& text) const
{
    // TODO: Create a queue with text and delays
    debug_log ("Playing after %1 seconds: %2", delay, text);
    if (! delay)
      play_now (text + "\n");    
    else
      Glib::signal_timeout().connect (sigc::bind<std::string> (
	sigc::mem_fun (this, &SynthesisProgram::play_now), text + "\n"), delay * 1000);
}

bool SynthesisProgram::play_now (const std::string& text) const
{
    debug_log ("Playing now: %1", text);
    write (prog_pipe, text.c_str(), text.size());
    return false;
}

void SynthesisProgram::terminate()
{
    if (prog_pipe) {
	close (prog_pipe);
	prog_pipe = 0;
    }
}
