/***************************************************************************
 *            TempSettingsManager.cc
 *
 *  Mon Nov 29 20:29:41 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "TempSettingsManager.h"

#include <boost/utility.hpp>

void TempSettingsManager::set_string (const std::string& group, const std::string& key, const std::string& value)
{
    std::string oldvalue;
    if (get_checked_string (group, key, oldvalue) && oldvalue == value) return;
    settings[group][key].value = value;
    signals[group][key].emit();
}

void TempSettingsManager::set_description (const std::string& group, const std::string& key, const std::string& value)
{
    settings[group][key].description = value;
}

void TempSettingsManager::set_hint (const std::string& group, const std::string& key, const std::string& value)
{
    settings[group][key].hint = value;
}

TempSettingsManager::signal_type TempSettingsManager::get_signal (const std::string& group, const std::string& key)
{
    return signals[group][key];
}

bool TempSettingsManager::get_checked_string (const std::string& group, const std::string& key, std::string& value) const
{
    SettingsGroupMap::const_iterator g;
    SettingsMap::const_iterator k;
    if ((g = settings.find (group)) != settings.end() &&
	(k = g->second.find (key)) != g->second.end()) {
	value = k->second.value;
	return true;
    }
    return false;
}

bool TempSettingsManager::delete_entry (const std::string& group, const std::string& key)
{
    SettingsGroupMap::iterator g;
    SettingsMap::iterator k;
    if ((g = settings.find (group)) != settings.end() &&
	(k = g->second.find (key)) != g->second.end()) {
	g->second.erase (k);
	return true;
    }
    return false;
}

// This shouldn't be used too often because it requires linear time
std::string TempSettingsManager::get_name (const std::string& group, const unsigned idx)
{
    SettingsGroupMap::const_iterator g;
    SettingsMap::const_iterator k;
    if ((g = settings.find (group)) != settings.end() &&
	(idx < g->second.size())) {
	return boost::next (g->second.begin(), idx)->first;
    }
    return "";
}
