/***************************************************************************
 *            TextStressBase.cc
 *
 *  Tue Feb  1 12:16:26 2005
 *  Copyright  2005  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "TextStressBase.h"
#include "debug.h"

#include <boost/scoped_array.hpp>

#include <fstream>

TextStressBase::TextStressBase (const std::string& file) 
{
    load (file);
}

void TextStressBase::load (const std::string& file) 
{
    std::ifstream istr (file.c_str());

    int pos;
    while (istr >> pos) {
	std::string data;
	istr >> data;
	long long unsigned md5 = get_md5 (data);
	if (accents.find (md5) != accents.end())
	  debug_log ("Insert into map has ambiguous md5 value: %1", data);
	else
	  accents.insert (std::make_pair (md5, pos));
    }
}

int TextStressBase::get_accent (const std::string& name) const 
{
    long long unsigned md5 = get_md5 (name);
    AccentMap::const_iterator i;
    if ((i = accents.find (md5)) != accents.end())
      return i->second;
    else
      debug_log ("Word not found: %1", name);
    return -1;
}

void TextStressBase::save_binary (const std::string& file) 
{
    FILE *f;
    unsigned size = accents.size();
    boost::scoped_array<long long unsigned> keys (new long long unsigned [size]);
    boost::scoped_array<unsigned char> acc (new unsigned char [size]);
    
    long long unsigned *keys_ptr = keys.get();
    unsigned char *acc_ptr = acc.get();
    AccentMap::const_iterator i;
    for (i = accents.begin(); i != accents.end(); ++i, ++keys_ptr, ++acc_ptr) {
	*keys_ptr = i->first;
	*acc_ptr = i->second;
    }
    
    f = fopen (file.c_str(), "wb");
    fwrite (&size, sizeof (size), 1, f);
    fwrite (keys.get(), sizeof (keys[0]), size, f);
    fwrite (acc.get(), sizeof (accents[0]), size, f);
    fclose (f);
}
