/***************************************************************************
 *            TransientSettings.h
 *
 *  Mon Nov 29 19:28:36 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _TRANSIENTSETTINGS_H
#define _TRANSIENTSETTINGS_H

#include "SettingsHelper.h"
#include "RegisteredSingleton.h"

#include <gtkmm/treemodelcolumn.h>
#include <gtkmm/liststore.h>
#include <libglademm/xml.h>

#include <boost/bind.hpp>
#include <boost/utility.hpp>
#include <boost/scoped_ptr.hpp>

namespace Gtk {
    class ComboBox;
    class CheckButton;
    class RadioButton;
    class ToggleToolButton;
}

class LanguageListModelColumns : public Gtk::TreeModelColumnRecord
{
  public:
    LanguageListModelColumns() { add(name); }

    Gtk::TreeModelColumn<Glib::ustring> name;
};

class TransientSettings : 
  public RegisteredSingleton<TransientSettings, 89>,
  public Settings 
{
  public:
    GenericSetting<bool> presentsolution;    
    GenericSetting<bool> usetimer;    
  
    TransientSettings();
    ~TransientSettings();

    // Language listboxes
    std::string get_first() const;
    std::string get_second() const;
    sigc::signal<void> signal_languages();
    template <class iter> void set_languages (const iter begin, const iter end, const std::string& first, const std::string& second);

    // Training mode
    bool is_audio() const;
    bool is_written() const;
    bool is_buttons() const;
    bool is_input() const;
    sigc::signal<void> signal_mode();
    void set_mode (const bool audio, const bool written, const bool buttons);

    // Session management
    bool is_testmode() const;
    bool is_resultsshown() const;
    void set_testmode (const bool value);
    void set_resultsshown (const bool value);
    sigc::signal<void> signal_testmode();
    sigc::signal<void> signal_resultsshown();

  private:
    void on_language_changed (const int which);
    void set_first (const std::string& value, const bool silent = false);
    void set_second (const std::string& value, const bool silent = false);
    void combo_set (Gtk::ComboBox* box, const std::string& value);
    bool combo_find (const std::string& value);

    void append_language (const std::string& lang);
    Gtk::ComboBox *firstlanguage;
    Gtk::ComboBox *secondlanguage;
    LanguageListModelColumns language_columns;
    Glib::RefPtr<Gtk::ListStore> language_store;
    sigc::signal<void> signal_languages_v;

    void on_presentwritten_changed();
    void set_written (const bool value, const bool silent = false);
    Gtk::CheckButton *presentwritten;

    void on_presentaudio_changed();
    void set_audio (const bool value, const bool silent = false);
    Gtk::CheckButton *presentaudio;

    void on_ask_changed();
    void set_buttons (const bool value, const bool silent = false);
    Gtk::RadioButton *askwritten;
    Gtk::RadioButton *askpick;

    sigc::signal<void> signal_mode_v;
};

template <class iter> void TransientSettings::set_languages (const iter begin, const iter end, const std::string& first, const std::string& second)
{
    language_store->clear();
    for_each (begin, end, bind (&TransientSettings::append_language, this, _1));
    
    if (language_store->children().empty()) return;
    set_first (first, true);
    set_second (second, true);
}

#endif /* _TRANSIENTSETTINGS_H */
