/***************************************************************************
 *            TwoDeData.h
 *
 *  Fri Jan 28 16:10:04 2005
 *  Copyright  2005  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _TWODEDATA_H
#define _TWODEDATA_H

#include <boost/multi_array.hpp>
#include <boost/shared_ptr.hpp>

#ifdef __cplusplus
extern "C"
{
#endif

class TwoDeData
{
  public:
    typedef boost::multi_array<double, 2>                 Data;
    typedef boost::shared_ptr<Data>                       DataPtr;
    typedef boost::multi_array_ref<double, 2>             DataRef;
    typedef DataRef::array_view<1>::type                  FlatRef;
    typedef boost::const_multi_array_ref<double, 2>       ConstDataRef;
    typedef ConstDataRef::const_array_view<1>::type       ConstFlatRef;

    static DataRef ref (Data& data);
    static ConstDataRef ref (const Data& data);
    static FlatRef row (Data& data, const unsigned num);
    static ConstFlatRef row (const Data& data, const unsigned num);
    static FlatRef col (Data& data, const unsigned num);
    static ConstFlatRef col (const Data& data, const unsigned num);
    static FlatRef flatten (Data& data);
    static FlatRef flatten (DataRef& data);
    static ConstFlatRef flatten (const Data& data);
    static ConstFlatRef flatten (const ConstDataRef& data);
    static DataPtr wrap (const ConstFlatRef& data);

    static DataPtr diff (const Data& data1, const Data& data2);
    static void gamma (Data& data, const double gamma);
    static void gamma (const Data& data1, Data& data2, const double gamma);
    static void normalize (Data& data);
    static void normalize (const Data& data1, Data& data2);
    static void normalize_neg (Data& data);
    static void normalize_neg (const Data& data1, Data& data2);
    static void normalize_log (Data& data, const double scale = 1.0);
    static void normalize_log (const Data& data1, Data& data2, const double scale = 1.0);
    static void normalize_log (Data& data, const double scale, const double max);
    static void normalize_log (const Data& data1, Data& data2, const double scale, const double max);
    static void clear (Data& data);
};

#ifdef __cplusplus
}
#endif

#endif /* _TWODEDATA_H */
