/***************************************************************************
 *            VocabCache.h
 *
 *  Sat Feb  5 13:55:59 2005
 *  Copyright  2005  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _VOCABCACHE_H
#define _VOCABCACHE_H

/** \file
 * Caches vocabulary scores.
 */

#include "RegisteredSingleton.h"

#include <boost/scoped_ptr.hpp>

#ifndef DOXYGEN_SHOULD_SKIP_THIS

namespace {
    class VocabCacheImpl;
}
    
#endif

/** Handle the caching of vocabulary scores.
 * For each vocabulary id, the date of the last correct answer
 * and a score calculated iteratively from the last answers
 * is saved.
 */

class VocabCache : public RegisteredSingleton<VocabCache, 70>
{
  public:
    VocabCache();
    ~VocabCache();
    
    void set_repeats (const unsigned new_decay);
    void set_expires (const unsigned days);
    void clear();
    void expire();
  
    void wrong (const unsigned id);
    void right (const unsigned id);
    double get_score (const unsigned id) const;
    double get_score_with_expires (const unsigned id) const;
    time_t get_expires (const unsigned id) const;
    
  private:
    boost::scoped_ptr<VocabCacheImpl> impl;
};

#endif /* _VOCABCACHE_H */
