/***************************************************************************
 *            VocabChecker.h - Check whether a guess is contained in the saved
 *                             translation
 *
 *  Sat Nov 20 20:12:35 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _VOCABCHECKER_H
#define _VOCABCHECKER_H

#include <glibmm/ustring.h>

#include <boost/tokenizer.hpp>

#include <string>
#include <list>

class VocabChecker 
{
  public:
    static bool check (const Glib::ustring& test, const Glib::ustring& base);
    
    // See friend VocabChecker std::for_each... some lines further down
#ifndef DOXYGEN_SHOULD_SKIP_THIS
    bool operator() (const std::string& base_token);
#endif

  private:
    VocabChecker (const Glib::ustring& test);
    bool get_result() const;
    bool does_match() const;
  
    typedef boost::tokenizer<boost::char_separator<char>, std::string::const_iterator, std::string> tokenizer;
    typedef boost::char_separator<char> separator;
  
    const std::string test_str;
    const separator test_sep;
    const tokenizer test_tokens;
    const unsigned test_len;

    std::list<std::string> base_part;
    std::list<bool> base_part_comment;
    bool base_comment;
    bool base_grammar;
    bool result;
  
    // This should work, but doesn't work due to a bug in the friend-template-namespace 
    // handling in gcc up to at least 3.4; operator() should be protected/private
    // See 
    //   "Befriending Templates" by Herb Sutter
    //   GCC-Bug 8355 at http://gcc.gnu.org/bugzilla/show_bug.cgi?id=8355
    
    //friend VocabChecker std::for_each<tokenizer::const_iterator, VocabChecker> 
    //  (tokenizer::const_iterator, tokenizer::const_iterator, VocabChecker);
};

#endif
