/***************************************************************************
 *            vocabparser.h
 *
 *  Wed Oct 13 11:41:50 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __VOCABPARSER_H
#define __VOCABPARSER_H

#include "data.h"
#include "VocabTest.h"

#include <glibmm/markup.h>

class VocabParser : public Glib::Markup::Parser
{
  public:
    VocabParser (DataChapters& chapters, VocabTypesPtr types, 
      VocabTest::LanguageSet& languages, VocabTest::PropertyMap& properties);
    virtual ~VocabParser();

  protected:
    //overrides:
    virtual void on_start_element (Glib::Markup::ParseContext& context,
      const Glib::ustring& element_name, const AttributeMap& attributes);
    virtual void on_end_element (Glib::Markup::ParseContext& context,
      const Glib::ustring& element_name) {};
    virtual void on_text (Glib::Markup::ParseContext& context, 
      const Glib::ustring& text);

  private:
    template <typename K, typename V, typename C> static V get_map (const std::map<K, V, C>& map, const K& key);
  
    DataChapters& chapters;
  
    VocabTypesPtr types;
  
    VocabTest::LanguageSet& languages;
    VocabTest::PropertyMap& properties;
  
    DataChapterPtr cur_chapter;
    DataSectionPtr cur_section;
    VocabularyPtr cur_vocab;

    Glib::ustring last_node;
    Glib::ustring last_lang;
    Glib::ustring last_property;
};

std::ostream& operator << (std::ostream& strm, const DataChapters& d);

// Templates

template <class iter> const std::string Vocabulary::print_trans (const iter& t) 
{
    return strmanip::ucompose ("        <translation lang=\"%1\">%2</translation>\n", 
      Glib::Markup::escape_text (t.first), Glib::Markup::escape_text (t.second));
}


#endif
