/***************************************************************************
 *            VocabSession.h
 *
 *  Tue Oct 19 14:12:14 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _VOCABSESSION_H
#define _VOCABSESSION_H

#include "VocabTest.h"
#include "SessionVocab.h"
#include "SessionFeeder.h"

#include <string>
#include <list>
#include <vector>

class VocabSession
{
  public:
    VocabSession (const SessionFeederPtr& feeder);
    
    void set_mode (const bool random);
    void set_interval (const unsigned num);
    void set_feed_continuous (const bool feed);

    // gives only an estimation
    unsigned int get_vocabs_left() const;

    void prepare_stats();
    unsigned get_stats_vocabs() const;
    unsigned get_stats_right() const;
    unsigned get_stats_wrong() const;
    unsigned get_stats_total() const;
    unsigned get_stats_unknown() const;

    bool start();
    bool advance();
    SessionVocabPtr get() const;
    SessionVocabPtr get_old() const;
    void defer (const bool force);
    bool feed();

  private:
    void defer_vocab (const bool ask_sooner);
    void update_stats (const SessionVocabPtr vocab);

    std::vector<SessionVocabPtr> vocabs;
    std::vector<SessionVocabPtr> sooner;
    std::vector<SessionVocabPtr> defered;
    std::set<SessionVocabPtr> history;
    SessionVocabPtr vocab_old;

    bool random_order;
    bool feed_continuous;
    unsigned interval;
  
    unsigned stats_right;
    unsigned stats_wrong;
    unsigned stats_unknown;

    SessionFeederPtr feeder;
};

typedef boost::shared_ptr<VocabSession> VocabSessionPtr;

#endif
