/***************************************************************************
 *            VocabTest.h
 *
 *  Tue Oct 19 14:12:14 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _VOCABTEST_H
#define _VOCABTEST_H

#include "data.h"
#include "VocabTypes.h"

#include <glibmm/ustring.h>

#include <boost/utility.hpp>
#include <boost/scoped_ptr.hpp>
#include <sigc++/signal.h>

#include <string>
#include <vector>
#include <set>

class VocabTest : boost::noncopyable
{
  public:
    VocabTest();
    ~VocabTest();
  
    void read_xml (const std::string& docname);
    void read_xml_data (const std::string& data);
    void write_xml();
    void write_xml (const std::string& docname);
    void write_xml (std::ostream& stream);
    const std::string& get_filename() const;
    const bool get_modified() const;
    void simple();
    void add_language (const std::string& language);
    const std::vector<std::string> get_languages() const;
    const VocabTypesPtr get_types() const;
    
    // chapters
    void insert_chapter (int chapter);
    void delete_chapter (int chapter);
    void copy_chapter (int source_chapter, int dest_chapter);
    const std::string get_chapter_name (int chapter) const;
    void set_chapter_name (int chapter, const std::string& newname);
    int get_chapter_num() const;

    // sections
    void insert_section (int chapter, int section);
    void delete_section (int chapter, int section);
    void copy_section (int source_chapter, int source_section, int dest_chapter, int dest_section);
    const std::string get_section_name (int chapter, int section) const;
    void set_section_name (int chapter, int section, const std::string& newname);
    int get_section_num (int chapter) const;

    static void normalize_indizes (std::vector<std::pair<int, int> >& data);
    
    // vocabs
    void insert_vocab (int chapter, int section, int index);
    void delete_vocab (int chapter, int section, int index);
    void delete_vocabs (int chapter, int section, std::vector<int> vocabs);
    void copy_vocab (int source_chapter, int source_section, int source_vocab, int dest_chapter, int dest_section, int dest_vocab);
    void copy_vocabs (int source_chapter, int source_section, std::vector<int> source_vocab, int dest_chapter, int dest_section, int dest_vocab);
    const std::string get_vocab (int chapter, int section, int index, const std::string& language) const;
    void set_vocab (int chapter, int section, int index, const std::string& language, const std::string& newtrans);
    const std::string get_type (int chapter, int section, int index) const;
    void set_type (int chapter, int section, int index, const std::string& newtype);
    unsigned get_id (int chapter, int section, int index) const;
    void set_id (int chapter, int section, int index, unsigned id);
    int get_vocab_num (int chapter, int section) const;

    // global file properties
    std::string get_property (const std::string& key) const;
    void set_property (const std::string& key, const std::string& value);

    typedef sigc::signal<void, int, int, int> type_signal_changed;
    typedef sigc::signal<void, int, int, int> type_signal_inserted;
    typedef sigc::signal<void, int, int, int> type_signal_deleted;
    typedef sigc::signal<void, int, int> type_signal_has_child_toggled;
    typedef sigc::signal<void, bool> type_signal_modified;
    
    type_signal_changed signal_on_changed();
    type_signal_inserted signal_on_inserted();
    type_signal_deleted signal_on_deleted();
    type_signal_has_child_toggled signal_on_has_child_toggled();
    type_signal_modified signal_on_modified();

    typedef std::set<std::string> LanguageSet;
    typedef std::map<std::string,std::string> PropertyMap;
    
  private:
    type_signal_changed signal_on_changed_v;
    type_signal_inserted signal_on_inserted_v;
    type_signal_deleted signal_on_deleted_v;
    type_signal_has_child_toggled signal_on_has_child_toggled_v;
    type_signal_modified signal_on_modified_v;

    void reset();

    void was_changed (const bool changed);
  
    void notify_insert_vocab (const int chapter, const int section, const int vocab);
    void notify_insert_section (const int chapter, const int section);
    void notify_insert_chapter (const int chapter);

    DataChapters chapters;

    VocabTypesPtr types;
   
    LanguageSet languages;

    PropertyMap properties;
  
    bool modified;
    std::string filename;
    
  friend std::ostream& operator << (std::ostream& strm, const VocabTest& d);
};

#endif /* _VOCABTEST_H */
