/***************************************************************************
 *            VocabTreeModel.h
 *
 *  Wed Oct 20 13:36:54 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _VOCABTREEMODEL_H
#define _VOCABTREEMODEL_H

#include <boost/shared_ptr.hpp>

#include <gtkmm/treemodel.h>
#include <gtkmm/treepath.h>
#include <gtkmm/treedragsource.h>
#include <gtkmm/treedragdest.h>

class VocabTest;
class VocabCache;
    
class VocabTreeModel : public Glib::Object, 
		       public Gtk::TreeModel,
		       public Gtk::TreeDragSource,
		       public Gtk::TreeDragDest
{
  public:
    typedef Gtk::TreeModelColumn<Glib::ustring> VocabColumn;
    typedef Gtk::TreeModelColumn<bool> BoolColumn;
    typedef std::vector<VocabColumn> TranslationVocabColumns;

    VocabTreeModel (const double fade, boost::shared_ptr<VocabTest> vocab_test);
    virtual ~VocabTreeModel();

    static Glib::RefPtr<VocabTreeModel> create (const double fade, boost::shared_ptr<VocabTest> vocab_test);

    void limit_to_branch (const iterator iter);
    void limit_to_branch (const Path path);
    void limit_to_root_branch();
    Path get_limit_branch() const { return root_path; };
    void set_limit_level (const int num) { depth = 2 - num; };
    int get_limit_level() const { return 2 - depth; };
    void get_vocab_addr (const Path& path, int &chapter, int &section, int &vocab) const;
    bool get_vocab_path (Path& path, const int chapter, const int section, const int vocab) const;

    VocabColumn& get_model_column (const int column);
    BoolColumn& get_model_column_bool (const int column);

    typedef enum {COLUMN_TYPE, COLUMN_TITLE, COLUMN_COLOR, COLUMN_SCORE, COLUMN_EXPIRED, COLUMN_EXPIRE_TIME, COLUMN_TRANSLATIONS_BASE};
  protected:
    // Overrides of TreeModel
    virtual Gtk::TreeModelFlags get_flags_vfunc() const;
    virtual int get_n_columns_vfunc() const;
    virtual GType get_column_type_vfunc (int index) const;
    virtual void get_value_vfunc (const TreeModel::iterator& iter, int column, Glib::ValueBase& value) const;
    bool iter_next_vfunc (const iterator& iter, iterator& iter_next) const;
    virtual bool iter_children_vfunc (const iterator& parent, iterator& iter) const;
    virtual bool iter_has_child_vfunc (const iterator& iter) const;
    virtual int iter_n_children_vfunc (const iterator& iter) const;
    virtual int iter_n_root_children_vfunc() const;
    virtual bool iter_nth_child_vfunc (const iterator& parent, int n, iterator& iter) const;
    virtual bool iter_nth_root_child_vfunc (int n, iterator& iter) const;
    virtual bool iter_parent_vfunc (const iterator& child, iterator& iter) const;
    virtual Path get_path_vfunc (const iterator& iter) const;
    virtual bool get_iter_vfunc (const Path& path, iterator& iter) const;

    virtual bool iter_is_valid (const iterator& iter) const;
  
    // Overrides of TreeDragSource
    virtual bool row_draggable_vfunc (const Path& path) const;
// This Method is implemented by default
//    virtual bool drag_data_get_vfunc (const Path& path, Gtk::SelectionData& selection_data) const;
    virtual bool drag_data_delete_vfunc (const Path& path);

    // Overrides of TreeDragSource
    virtual bool drag_data_received_vfunc (const Path& dest, const Gtk::SelectionData& selection_data);
    virtual bool row_drop_possible_vfunc (const Path& dest, const Gtk::SelectionData& selection_data) const;
    
private:
    bool iters_are_equal (const iterator& iter1, const iterator& iter2) const;
    void get_root_iterator (iterator& iter) const;
    void get_root_vocab_path (Path& path, const int chapter, const int section, const int vocab) const;

    void on_test_changed (const int chapter, const int section, const int index);
    void on_test_inserted (const int chapter, const int section, const int index);
    void on_test_deleted (const int chapter, const int section, const int index);
    void on_test_has_child_toggled (const int chapter, const int section);

    ColumnRecord columns_record;

    VocabColumn type_column;
    VocabColumn title_column;
    VocabColumn colorname_column;
    VocabColumn score_column;
    BoolColumn expired_column;
    VocabColumn expire_time_column;
    TranslationVocabColumns translation_columns;

    iterator root_iterator;
    Path root_path;

    boost::shared_ptr<VocabTest> vocab_test;
    const VocabCache& vocab_cache;

    int depth;
    int stamp;
    const double fade;
};

#endif /* _VOCABTREEMODEL_H */
