/***************************************************************************
 *            VocabTypes.cc
 *
 *  Thu Dec  9 11:25:15 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include "VocabTypes.h"

#include "langsupport.h"

#include <boost/utility.hpp>

#include <glib.h>

#include <sstream>

/* in all the routines are some workarounds to have a message saying that there
 * are no predefined vocab types. This is because of crashes with empty 
 * histories. If Gtk gets fixed it can be taken out. TODO 
 */
void VocabTypes::add_usage (const std::string& type)
{
    if (type == "") return;
    UsageMap::iterator i;
    if ((i = usage.find (type)) == usage.end()) {
	i = usage.insert (std::make_pair (type, 1)).first;
	if (usage.size() == 1)
	  signal_on_changed_v.emit (0);
	else
	  signal_on_inserted_v.emit (distance (usage.begin(), i));
    } else {
	i->second++;
	signal_on_changed_v.emit (distance (usage.begin(), i));
    }	
}

void VocabTypes::del_usage (const std::string& type)
{
    if (type == "") return;
    UsageMap::iterator i;
    if ((i = usage.find (type)) == usage.end()) {
	g_critical ("Trying do delete vocab type %s, which is not in the map.", type.c_str());
    } else {
	if (i->second == 1) {
	    unsigned oldpos = distance (usage.begin(), i);
	    usage.erase (i);
	    if (usage.size() == 0)
	      signal_on_changed_v.emit (0);
	    else
	      signal_on_deleted_v.emit (oldpos);
	} else {
	    i->second--;
	    signal_on_changed_v.emit (distance (usage.begin(), i));
	}
    }
}

/* The next two functions get highly inefficient for more than a couple of
 * strings; but this shouldn't be the case for vocabulary types
 */
std::string VocabTypes::get_name (const unsigned idx) const
{
    if (usage.size() == 0)
      return _("No vocabulary types available");
    else
      return (boost::next (usage.begin(), idx))->first;
}

std::string VocabTypes::get_num (const unsigned idx) const
{
    std::ostringstream s;
    if (usage.size() != 0)
      s << boost::next (usage.begin(), idx)->second;
    return s.str();
}

unsigned VocabTypes::size() const
{
    if (usage.size() == 0)
      return 1;
    else
      return usage.size();
}

VocabTypes::type_signal_changed VocabTypes::signal_on_changed()
{
  return signal_on_changed_v;
}

VocabTypes::type_signal_inserted VocabTypes::signal_on_inserted()
{
  return signal_on_inserted_v;
}

VocabTypes::type_signal_deleted VocabTypes::signal_on_deleted()
{
  return signal_on_deleted_v;
}
