/***************************************************************************
 *            VocabTypes.h
 *
 *  Thu Dec  9 11:26:25 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _VOCABTYPES_H
#define _VOCABTYPES_H

#include <boost/shared_ptr.hpp>

#include <string>
#include <map>
#include <sigc++/signal.h>

class VocabTypes;
    
typedef boost::shared_ptr<VocabTypes> VocabTypesPtr;

class VocabTypes
{
  public:
    void add_usage (const std::string& type);
    void del_usage (const std::string& type);
  
    void clear();
    unsigned size() const;

    std::string get_name (const unsigned idx) const;
    std::string get_num (const unsigned idx) const;
  
    typedef sigc::signal<void, int> type_signal_changed;
    typedef sigc::signal<void, int> type_signal_inserted;
    typedef sigc::signal<void, int> type_signal_deleted;
    
    type_signal_changed signal_on_changed();
    type_signal_inserted signal_on_inserted();
    type_signal_deleted signal_on_deleted();

  protected:
    type_signal_changed signal_on_changed_v;
    type_signal_inserted signal_on_inserted_v;
    type_signal_deleted signal_on_deleted_v;

  private:
    typedef std::map<std::string, unsigned> UsageMap;
    UsageMap usage;
};

#endif /* _VOCABTYPES_H */
