/***************************************************************************
 *            VocabTypesModel.h
 *
 *  Thu Dec  9 15:57:32 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _VOCABTYPESMODEL_H
#define _VOCABTYPESMODEL_H

#include "VocabTypes.h"

#include <gtkmm/treemodel.h>

class VocabTypesModel : 
  public Glib::Object, 
  public Gtk::TreeModel
{
  public:
    typedef Gtk::TreeModelColumn<Glib::ustring> TextColumn;
    typedef Gtk::TreeModelColumn<Glib::ustring> NumColumn;

    VocabTypesModel (const VocabTypesPtr types);
    virtual ~VocabTypesModel();
  
    TextColumn& get_column_text();
    TextColumn& get_column_num();

    static Glib::RefPtr<VocabTypesModel> create (const VocabTypesPtr types);

  protected:
    // Overrides of TreeModel
    virtual Gtk::TreeModelFlags get_flags_vfunc() const;
    virtual int get_n_columns_vfunc() const;
    virtual GType get_column_type_vfunc (int index) const;
    virtual void get_value_vfunc (const TreeModel::iterator& iter, int column, Glib::ValueBase& value) const;
    bool iter_next_vfunc (const iterator& iter, iterator& iter_next) const;
    virtual bool iter_children_vfunc (const iterator& parent, iterator& iter) const;
    virtual bool iter_has_child_vfunc (const iterator& iter) const;
    virtual int iter_n_children_vfunc (const iterator& iter) const;
    virtual int iter_n_root_children_vfunc() const;
    virtual bool iter_nth_child_vfunc (const iterator& parent, int n, iterator& iter) const;
    virtual bool iter_nth_root_child_vfunc (int n, iterator& iter) const;
    virtual bool iter_parent_vfunc (const iterator& child, iterator& iter) const;
    virtual Path get_path_vfunc (const iterator& iter) const;
    virtual bool get_iter_vfunc (const Path& path, iterator& iter) const;

    virtual bool iter_is_valid (const iterator& iter) const;
  
private:
    void on_types_changed (const unsigned idx);
    void on_types_inserted (const unsigned idx);
    void on_types_deleted (const unsigned idx);

    ColumnRecord columns_record;

    TextColumn text_column;
    NumColumn num_column;

    VocabTypesPtr types;

    int stamp;
};

#endif /* _VOCABTYPESMODEL_H */
