/***************************************************************************
 *            XMLSettingsManager.cc
 *
 *  Mon Nov 29 20:29:41 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "XMLSettingsManager.h"
#include "XMLSettingsParser.h"

#include "ProgramPaths.h"

#include "debug.h"

#include <glibmm/fileutils.h>
#include <glibmm/miscutils.h>

#include <fstream>

XMLSettingsManager::XMLSettingsManager (const std::string& configname) :
  filename (Glib::build_filename (ProgramPaths::home(), configname))
{
    debug_log ("Reading configuration data from %1", filename);
    try {
	XMLSettingsParser parser (*this);
	Glib::Markup::ParseContext context (parser);
	context.parse (Glib::file_get_contents (filename));
	context.end_parse();
    } catch (...) {};
};

XMLSettingsManager::~XMLSettingsManager()
{
    debug_log ("Writing configuration data to %1", filename);
    std::ofstream file (filename.c_str());
    if (!file) return;
    file << *this;
};
