/***************************************************************************
 *            vocabparser.cc
 *
 *  Tue Oct 12 17:32:22 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "XMLSettingsParser.h"
#include "XMLSettingsManager.h"

#include "stringhelper.h"

#include <sstream>

XMLSettingsParser::XMLSettingsParser (XMLSettingsManager& manager)
  : manager (manager)
{
}

XMLSettingsParser::~XMLSettingsParser()
{
}

template <typename K, typename V, typename C> V XMLSettingsParser::get_map 
  (const std::map<K, V, C>& map, const K& key)
{
    typename std::map<K, V, C>::const_iterator i;
    if ((i = map.find (key)) != map.end())
      return i->second;
    else
      return V();
}

void XMLSettingsParser::on_start_element (Glib::Markup::ParseContext& context,
  const Glib::ustring& element_name, const AttributeMap& attributes)
{
    if (element_name == "group") {
	cur_group = get_map (attributes, Glib::ustring ("name"));
    } else if (element_name == "key") {
	cur_key = get_map (attributes, Glib::ustring ("name"));
    }
}

void XMLSettingsParser::on_text (Glib::Markup::ParseContext& context, 
  const Glib::ustring& text)
{
    Glib::ustring trimmed = trim_whitespace (text);
    if (! trimmed.empty())
      manager.set_string (cur_group, cur_key, trimmed);
}

const std::string XMLSettingsManager::print_group (const SettingsGroupMap::value_type& t)
{
    std::ostringstream strm;
    strm << "  <group name=\"" << Glib::Markup::escape_text (t.first) << "\">\n";
    std::transform (t.second.begin(), t.second.end(), 
      std::ostream_iterator<std::string> (strm), XMLSettingsManager::print_key);
    strm << "  </group>\n";
    return strm.str();
}

const std::string XMLSettingsManager::print_key (const SettingsMap::value_type& t)
{
    std::string value;
    if (t.second.hint != "") value +=
      "    <!-- " + Glib::Markup::escape_text (t.second.hint) + "-->\n";
    if (t.second.description != "") value +=
      "    <!-- " + Glib::Markup::escape_text (t.second.description) + "-->\n";
    value +=
      "    <key name=\"" +
	Glib::Markup::escape_text (t.first) +
      "\">" +
	Glib::Markup::escape_text (t.second.value) +
      "</key>\n";
    return value;
}

std::ostream& operator << (std::ostream& strm, const XMLSettingsManager& d) { 
    strm << "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n";
    strm << "<!DOCTYPE config SYSTEM \"config.dtd\">\n";
    strm << "<config>\n";
    std::transform (d.settings.begin(), d.settings.end(), 
      std::ostream_iterator<std::string> (strm), XMLSettingsManager::print_group);
    strm << "</config>\n";
    return strm; 
};
