/***************************************************************************
 *            data.h
 *
 *  Tue Oct 12 18:30:27 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _DATA_H
#define _DATA_H

#include "ucompose.hpp"

#include "VocabTypes.h"

#include <boost/shared_ptr.hpp>

#include <string>
#include <vector>
#include <map>
#include <iostream>

class Vocabulary;
    
typedef boost::shared_ptr<Vocabulary> VocabularyPtr;
typedef std::vector<VocabularyPtr> Vocabularies;

class Vocabulary
{
  public:
    Vocabulary (VocabTypesPtr types, const unsigned id);
    Vocabulary (VocabTypesPtr types);
    virtual ~Vocabulary();

    Vocabulary (const Vocabulary& other);
    const Vocabulary& operator= (const Vocabulary& other);

    std::string get_translation (const std::string& which) const; 
    void set_translation (const std::string& which, const std::string& what); 

    const std::string& get_type() const; 
    void set_type (const std::string& what); 
  
    unsigned get_id() const { return id; }; 
    void set_id (const unsigned new_id) { id = new_id; }; 
    void new_id(); 

  private:
    void init();
    void copy (const Vocabulary& other);
  
    template <class iter> static void dump_trans (const iter& i);
    template <class iter> static const std::string print_trans (const iter& t);
    
    typedef std::map<const std::string,std::string> LanguageMap;
    LanguageMap translations;
    unsigned id;
    std::string type;
    VocabTypesPtr types;
  
  friend std::ostream& operator << (std::ostream& strm, const VocabularyPtr& d);
  friend class VocabParser;
};

class DataSection;
typedef boost::shared_ptr<DataSection> DataSectionPtr;
typedef std::vector<DataSectionPtr> DataSections;

class DataSection
{
  public:
    DataSection() {};
    DataSection (const std::string title) : title(title) {};

    DataSection (const DataSection& other);
    const DataSection& operator= (const DataSection& other);
	
    const std::string& get_title() const; 
    void set_title (const std::string& what); 
  
    Vocabularies vocabs;
	
  private:
    void copy (const DataSection& other);
    void copy_vocab (const VocabularyPtr& other);
  
    std::string title;
};

class DataChapter;
typedef boost::shared_ptr<DataChapter> DataChapterPtr;
typedef std::vector<DataChapterPtr> DataChapters;
    
class DataChapter
{
  public:
    DataChapter() : difficulty (1) {};
    DataChapter (const std::string title, const int difficulty) : 
      difficulty (difficulty), title (title) {};

    DataChapter (const DataChapter& other);
    const DataChapter& operator= (const DataChapter& other);
	  
    const std::string& get_title() const; 
    void set_title (const std::string& what); 
  
    DataSections sections;

    int difficulty;

  private:
    void copy (const DataChapter& other);
    void copy_section (const DataSectionPtr& other);
  
    std::string title;
};

// Templates

template <class iter> void Vocabulary::dump_trans (const iter& i) {
    std::cerr << "  " << i.first << ": " << i.second << std::endl;
}

// Inline

/** Set a new title.
 * \param what New title
 */

inline void DataSection::set_title (const std::string& what) 
{
    title = what;
}

/** Get the current title.
 * \return Current title
 */

inline const std::string& DataSection::get_title() const
{
    return title;
}

/** Set a new title.
 * \param what New title
 */

inline void DataChapter::set_title (const std::string& what) 
{
    title = what;
}

/** Get the current title.
 * \return Current title
 */

inline const std::string& DataChapter::get_title() const
{
    return title;
}

/** Set a new type.
 * The old type is unregistered and the new type registered with the
 * responsible VocabTypes instance.
 * \param what New type
 */

inline void Vocabulary::set_type (const std::string& what) 
{
    types->del_usage (type);
    type = what;
    types->add_usage (type);
}

/** Get the current type.
 * \return Current type
 */

inline const std::string& Vocabulary::get_type() const
{
    return type;
}


#endif /* _DATA_H */
