/***************************************************************************
 *            exception.cc
 *
 *  Tue Oct 12 20:40:28 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/** \file
 * Basic exception class.
 */

#include "exception.h"

#include <string>

std::string error_prep(const std::string & message,
  const std::string & reason)
{
    return message + ": " + reason;
}

const std::string empty_string;

class ApplicationError::m_imp
{
  public:
    std::string m_what;
    std::string m_message;
    std::string m_reason;
};

/** Construct an exception with only a reason.
 * The error message will consist solely of the error reason.
 * \param reason Error reason
 */

ApplicationError::ApplicationError (const std::string & reason)
{
    try {
        m_imp_ptr.reset (new m_imp);
        m_imp_ptr->m_what = reason;
        m_imp_ptr->m_reason = reason;
    } catch (...) { 
	m_imp_ptr.reset(); 
    }
}

/** Construct an exception with a message and a reason.
 * The error message will consist of the message, followed by the reason.
 * \param message Error message
 * \param reason Error reason
 */

ApplicationError::ApplicationError (const std::string & message, const std::string & reason)
{
    try {
        m_imp_ptr.reset (new m_imp);
        m_imp_ptr->m_what = error_prep (message, reason);
        m_imp_ptr->m_message = message;
        m_imp_ptr->m_reason = reason;
    } catch (...) { 
	m_imp_ptr.reset(); 
    }
}

ApplicationError::~ApplicationError() throw()
{
}

/** Return a textual repesentation of error.
 * \return Formated message
 */

const char * ApplicationError::what() const throw()
{
    return m_imp_ptr.get() == 0 ? empty_string.c_str() : 
      m_imp_ptr->m_what.c_str();
}
