/***************************************************************************
 *            functions.cc
 *
 *  Mon Feb  7 19:30:55 2005
 *  Copyright  2005  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/** \file
 * Defines some commonly used function classes.
 */
 
#include "functions.h"

#include <sstream>
#include <iomanip>
#include <iostream>

namespace func
{
    
    /** Transform a color value given by 3 numbers in the range [0,1] to a 
     * hexadecimal equivalent unterstood by X.
     * \param r Red component
     * \param g Green component
     * \param b Blue component
     * \return String of the form \c #rrrrggggbbbb
     */
    
    std::string x_color (const double r, const double g, const double b) 
    {
	std::ostringstream s;
	s << "#" << std::hex << std::setfill ('0') << 
	  std::setw (4) << static_cast<unsigned> (r * 0xffff) << 
	  std::setw (4) << static_cast<unsigned> (g * 0xffff) << 
	  std::setw (4) << static_cast<unsigned> (b * 0xffff);
	return s.str();	
    }
}

/** Activates a temporary locale for LC_ALL.
 \param new_locale Name of the new locale
  */
  
TempLocale::TempLocale (const std::string& new_locale) :
  old_locale (setlocale (LC_ALL, NULL))
{
    setlocale (LC_ALL, new_locale.c_str());
}
	
/** Restores the original locale for LC_ALL.
 */
 
TempLocale::~TempLocale()
{
    setlocale (LC_ALL, old_locale.c_str());
}
