#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "MainWindow.h"
#include "GlobalXML.h"
#include "TextStressBase.h"
#include "SynthesisAnalysis.h"
#include "CmdLine.h"
#include "ProgramPaths.h"
#include "TrayIcon.h"

#include "langsupport.h"

#include "debug.h"

#ifdef ENABLE_NLS
#    include <libintl.h>
#endif

#include <libgnomecanvasmm/init.h>

#include <gtkmm/main.h>
#include <gtkmm/accelmap.h>
#include <gtkmm/liststore.h>

#include <iostream>

gchar* convert_text;
gchar* convert_binary;
gboolean start_quiet = false;

GOptionEntry options[] = 
{
    { "convert-text", 0, 0, G_OPTION_ARG_FILENAME, &convert_text, N_("Convert text stressbase to binary"), N_("NAME") },
    { "binary-name", 0, 0, G_OPTION_ARG_FILENAME, &convert_binary, N_("New name for the binary stressbase"), N_("NAME") },
    { "start-quiet", 0, 0, G_OPTION_ARG_NONE, &start_quiet, N_("Start quietly in timer mode"), NULL },
    { NULL }
};

int main (int argc, char *argv[])
{
    // We need the program path for Win32
    CmdLine::set (argc, argv);
    
#ifdef ENABLE_NLS
    bindtextdomain (PACKAGE, ProgramPaths::locale().c_str());
    textdomain (PACKAGE);
#endif

    GError *error = NULL;
    GOptionContext* context = g_option_context_new (_("[file] - interactivly test vocabularies"));
    g_option_context_add_main_entries (context, options, ProgramPaths::domain().c_str());
    g_option_context_add_group (context, gtk_get_option_group (TRUE));
    
    debug_log_setup_cmdline (context);
    
    g_option_context_parse (context, &argc, &argv, &error);    
    
    if (convert_text) {
	if (! convert_binary) {
	    std::cerr << _("You must provide a new filename for the binary stressbase with --binary-name\n");
	    exit (1);
	}
	TextStressBase s (convert_text);
	s.save_binary (convert_binary);
	exit (0);
    }

    Gnome::Canvas::init();
    Gtk::Main m(&argc, &argv);

    // Reset with the remaining parameters
    CmdLine::set (argc, argv);

    debug_log_init_cmdline();

//    Gtk::AccelMap::load ("/home/mh21/.vocabtrainaccels");

    MainWindow* window;
    GlobalXML::get().get_glade()->get_widget_derived ("MainWindow", window);
    if (start_quiet)
      window->quiet_timer_mode();
    window->show();
    m.run();
    
//    Gtk::AccelMap::save ("/home/mh21/.vocabtrainaccels");
    
    delete window; // This gives us destructors in MainWindow-related objects
    
    return 0;
}
