/***************************************************************************
 *            stringhelper.h
 *
 *  Wed Oct 13 11:41:50 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#ifndef _STRINGHELPER_H
#define _STRINGHELPER_H

/** \file 
 * Various string functions.
 */

#include <glibmm/ustring.h>

#include <string>
#include <sstream>

/** Convert a value of type \c T from a std::string using a std::istringstream.
 * \param t Location for the return value
 * \param s String to read from
 * \param f std::ios_base stream modifier
 * \return whether the conversion was successful
 */

template <class T> 
bool from_string (T& t, const std::string& s, std::ios_base& (*f)(std::ios_base&))
{
   std::istringstream iss (s);
   return ! (iss >> f >> t).fail();
}

/** Convert a value of type \c T from a string of type \c U using a std::istringstream.
 * \param s String to read from
 * \param d Default value if the conversion is not successful
 * \param f std::ios_base stream modifier
 * \return converted value
 */

template <class T,class U> 
T from_string (const U& s, const T d,
   std::ios_base& (*f)(std::ios_base&) = std::dec)
{
    T t;
    std::istringstream iss (s);
    return (iss >> f >> t).fail() ? d : t;
}

Glib::ustring trim_whitespace (const Glib::ustring& text);

bool is_only_whitespace (const Glib::ustring& text);

std::string format_time (const std::string& format, const time_t& time);

#endif /* _SUPPORT_H */
