/***************************************************************************
 *            support.cc
 *
 *  Wed Oct 13 19:59:31 2004
 *  Copyright  2004  Michael Hofmann
 *  mh21@gmx.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
/** \file
 * Some utility functions that didn't fit somewhere else.
 */
 
#include "support.h"
#include "debug.h"

#include <gtkmm/treeview.h>
#include <gtk/gtktreeview.h>

#include <ctime>


/** Look whether there is a selection in a GtkTreeView. 
 * It doesn't use gtkmm because of sporadic segfaults while handling 
 * NULL GtkTreePath.
 */
 
bool cursor_visible (Gtk::TreeView* tv) {
    GtkTreePath* pa;
    GtkTreeViewColumn* co;
    if (tv) {
	gtk_tree_view_get_cursor (tv->gobj(), &pa, &co);
	if (pa) {
	    gtk_tree_path_free (pa);
	    return true;
	}
    }
    return false;
}

/** Generate a random number.
 * The random number generator is initialized before the first use
 * automatically.
 * \param n Maximum value or 0 for unclipped
 */
 
unsigned generic_random (const unsigned n) {
    // Funny way to execute srand only once
    static const bool initialized = (srand (time (NULL)), true);
    ignore_unused_variable_warning (initialized);
    
    const unsigned rand_val = rand();
    const unsigned cut_rand = n ? (rand_val % n) : rand_val;
    debug_log ("Initial %1, clipped by %2, gives %3", rand_val, n, cut_rand);
    return cut_rand;
}
